;***********************************************************************
;*                                                                     *
;*                    8 Bit Sound Player and Recorder                  *
;*     Header file with Sound Card basic input,output functions        *
;*              You also have to use BND_MAC.H header file             *
;*                                                                     *
;*                   Copyright(c) Bognr Andrs  2005                  *
;*                                                                     *
;*  Uses 8 bit DMA, Sound Blaster 16 or Pro ,PORT 220H , DMA1, IRQ5    *
;*  Simple user interface,  written for DOS, WIN3.XX, WIN95            *
;***********************************************************************

flag db 0                ;interrupt was detected if =1
base dw 220h             ;Sound Card base address (for mixer port)
n_exist db 'Sound Card is not exist',10,13,0

;------------------------Interrupt routine for IRQ5------------------------
; Sound Card hardware interrupt routine
; It sets the variable [flag] to "1" to sign that interrupt happened
inte:
push ax                  ;saving the CPU registers
push ds
push dx
mov ax,cs                ;CS=DS
mov ds,ax
mov dx,22Eh              ;read the Sound Card state port
in al,dx
mov al,1                 ;interrupt mark flag=1
mov [flag],al
cli
mov al,20h               ;End Of Interrupt (EOF) command to IT controller
out 20h,al
sti
pop dx                   ;pop the CPU registers from the stack
pop ds
pop ax
iret

;---------------------------Initialize the Sound Card---------------------
; Resets the Sound Card and controls if the Sound Cars exist
initsound:
mov dx,226h              ;reset the sound card with ramp sign
mov al,1                 ;to the reset port ( /-\ )
out dx,al
delay 10,10
mov al,0
out dx,al
delay 10,100             ;critical delay !
mov dx,22Eh
in al,dx                 ;test the answer from the Card
test al,80H              ;Is the data output register full?
jz serr
mov dx,22Ah
in al,dx
cmp al,0AAh              ;Card has to give '0AAH' after reset
jnz serr
mov al,0e0h              ;test the dsp: Warning! It can crash here.
call writedsp
mov al,88h
call writedsp
call readdsp
cmp al,77h               ;DSP has to negate the input data
jnz serr
ret
; ERROR reporting and program aborting if Sound Card is not exist or
; does'n meet the requirements
serr:                    ;Sound Card error handling
putbs 0,n_exist          ;Error reporting to the CRT
puteol                   ;Put EOL (10,13) to the CRT
end                      ;Program termination macro

;---------------------------Write data to the DSP-------------------------
; Wait for the empty instruction register by data polling
; and write the contain of the AL register to the instruction port
writedsp:
push ax                  ;IN: al=data
varas1:                  ;data polling cycle
mov dx,22Ch              ;read the state port
in al,dx
test al,80H              ;If bit7 is set then write the instruction
jnz varas1
pop ax
out dx,al                ;write the instruction given by AL register
ret                      ;to the instruction port

;-------------------------------Read from the DSP-------------------------
; Wait for the data by data polling and read the data from the DSP port
readdsp:
mov dx,22Eh              ;OUT: al=data
in al,dx                 ;Read the state port
test al,80h              ;if bit7 is set then read the data port
jz readdsp
mov dx,22Ah
in al,dx                 ;Read the data port to AL register
ret

;----------------------------Switch on the speaker-----------------------
speakeron:
mov al,0D1h          ;0D1h is a DSP instruction, it switches on the spkr.
call writedsp            ;Call function writedsp ,the parameter is given
ret                      ;in register AL

;------------------------------Switch off the speaker--------------------
speakeroff:
mov al,0D3h          ;0D3h is a DSP instruction, it switches off the spkr.
call writedsp            ;Call function writedsp ,the parameter is given
ret                      ;in register AL

;-------------------------------Stop using the DMA------------------------
dmastop:
mov al,0D0h          ;0D0h is a DSP instruction, it stops the DMA operation
call writedsp
ret

;------------------------------Continue using the DMA----------------------
dmacontinue:
mov al,0D4h       ;0D4h is a DSP instruction, it enables the DMA operation
call writedsp
ret

;--------------------------------Write the mixer---------------------------
; Writes the instructions to the mixer registers, you have to give the
; mixer register number in the AH register and the instruction in the AL reg.
mixer:
mov dx,[base]            ;IN: ah=register  al=data
add dx,04h
push ax
mov al,ah
out dx,al                ;write the ordinary number of the mixer register
pop ax                   ;to port 224H (for example)
inc dx                   ;increment port no.
out dx,al                ;write the instruction to the mixer data register
ret

setmixer:;------------------------Set the mixer----------------------------
;Initialize the mixer and write a basic setting to them
;set the Sound Blaster Pro mixer registers
call initsound
mov ax,0000h             ;mixer reset
call mixer
mov ax,0488h             ;DSP output volume  =medium
call mixer
mov ax,0600h             ;FM output =00h
call mixer
mov ax,0A08h             ;Microphone volume  =medium
call mixer
mov ax,0E20h             ;Analog output control 
call mixer               ;filter=disabled, Mono D/A mode
mov ax,2288h             ;General Volume control  =medium
call mixer
mov ax,2688h             ;FM synthezer volume =medium
call mixer
mov ax,2888h             ;CD volume =medium
call mixer
mov ax,2E88h             ;Line input volume =medium
call mixer
;We also have to set the Sound Blaster 16 mixer registers because
;in case of SB16 the SBPro settings can be ignored
mov ax,3080h             ;Left General volume  =medium
call mixer
mov ax,3180h             ;Right General volume =medium
call mixer
mov ax,3280h             ;DSP output left channel  =medium
call mixer
mov ax,3380h             ;DSP output right channel  =medium
call mixer
mov ax,3480h             ;MIDI volume left channel  =medium
call mixer
mov ax,3580h             ;MIDI volume right channel =medium
call mixer
ret